from pydantic import BaseModel 
from sqlalchemy import Column, String, Integer

from poindexter.models import Base


class KeywordCreate(BaseModel):
  value: str


class Keyword(KeywordCreate):
  id: int
  class Config:
    orm_mode = True


class KeywordModel(Base):
  __tablename__ = 'keywords'
  id = Column(Integer, primary_key=True, index=True)
  value = Column(String)



from enum import Enum
from typing import Optional, List
from datetime import datetime

import sqlalchemy

from pydantic import BaseModel 
from sqlalchemy import Column, String, Integer, DateTime, ForeignKey, Table
from sqlalchemy.orm import relationship

from poindexter.models import Base
from poindexter.models.CSE import CSE
from poindexter.models.APIKey import APIKey
from poindexter.models.Profile import Profile
from poindexter.models.Report import Report
from poindexter.models.Plugin import Plugin


Job_Plugin_Table = Table(
  'job_plugin_association',
  Base.metadata,
  Column('job_id', ForeignKey('jobs.id')),
  Column('plugin_id', ForeignKey('plugins.id'))
)


class JobStatus(Enum):
  CREATED = "CREATED"
  RUNNING = "RUNNING"
  ERROR = "ERROR"
  FINISHED = "FINISHED"


class JobCreate(BaseModel):
  name: str
  profile_id: int
  cse_id: int
  key_id: int


class JobUpdate(BaseModel):
  status: JobStatus


class Job(JobCreate):
  id: int
  status: JobStatus
  created: datetime
  profile: Profile
  cse: CSE
  key: APIKey
  report_id: Optional[int]
  report: Optional[Report]
  plugins: List[Plugin]
  class Config:
    orm_mode = True


class JobModel(Base):
  __tablename__ = 'jobs'
  id = Column(Integer, primary_key=True, index=True)
  name = Column(String)
  status = Column(sqlalchemy.Enum(JobStatus), default=JobStatus.CREATED)
  created = Column(DateTime, default=lambda: datetime.now())
  profile_id = Column(Integer, ForeignKey("profiles.id"))
  cse_id = Column(Integer, ForeignKey("cses.id"))
  key_id = Column(Integer, ForeignKey("keys.id"))
  report_id = Column(Integer, ForeignKey("reports.id"), nullable=True)
  profile = relationship("ProfileModel")
  cse = relationship("CSEModel")
  key = relationship("APIKeyModel")
  report = relationship("ReportModel")
  plugins = relationship("PluginModel", secondary=Job_Plugin_Table)



from typing import List

from pydantic import BaseModel
from sqlalchemy import Column, String, Integer, Table, ForeignKey
from sqlalchemy.orm import relationship

from poindexter.models import Base
from poindexter.models.Domain import Domain


CSE_Domain_Table = Table(
  'cse_domain_association',
  Base.metadata,
  Column('cse_id', ForeignKey('cses.id')),
  Column('domain_id', ForeignKey('domains.id'))
)


class CSECreate(BaseModel):
  name: str
  cx: str
  description: str


class CSE(CSECreate):
  id: int
  domains: List[Domain] = []
  class Config:
    orm_mode = True


class CSEModel(Base):
  __tablename__ = 'cses'
  id = Column(Integer, primary_key=True, index=True)
  name = Column(String)
  cx = Column(String)
  description = Column(String)
  domains = relationship("DomainModel", secondary=CSE_Domain_Table)



from typing import List

from pydantic import BaseModel 
from sqlalchemy import Column, String, Integer, Table, ForeignKey
from sqlalchemy.orm import relationship

from poindexter.models import Base
from poindexter.models.Link import Link
from poindexter.models.PluginOutput import PluginOutput


Report_Link_Table = Table(
  'report_link_association',
  Base.metadata,
  Column('report_id', ForeignKey('reports.id')),
  Column('link_id', ForeignKey('links.id'))
)

Report_Output_Table = Table(
  'report_output_association',
  Base.metadata,
  Column('report_id', ForeignKey('reports.id')),
  Column('output_id', ForeignKey('outputs.id'))
)


class ReportCreate(BaseModel):
  name: str


class Report(ReportCreate):
  id: int
  links: List[Link]
  outputs: List[PluginOutput]
  class Config:
    orm_mode = True


class ReportModel(Base):
  __tablename__ = 'reports'
  id = Column(Integer, primary_key=True, index=True)
  name = Column(String)
  links = relationship("LinkModel", secondary=Report_Link_Table)
  outputs = relationship("PluginOutputModel", secondary=Report_Output_Table)



from typing import List

from pydantic import BaseModel 
from sqlalchemy import Column, String, Integer, ForeignKey, Table
from sqlalchemy.orm import relationship

from poindexter.models import Base
from poindexter.models.Tag import Tag


Link_Tag_Table = Table(
  'link_tag_association',
  Base.metadata,
  Column('link_id', ForeignKey('links.id')),
  Column('tag_id', ForeignKey('tags.id'))
)


class LinkCreate(BaseModel):
  value: str
  screenshot: str


class Link(LinkCreate):
  id: int
  tags: List[Tag]
  class Config:
    orm_mode = True


class LinkModel(Base):
  __tablename__ = 'links'
  id = Column(Integer, primary_key=True, index=True)
  value = Column(String)
  screenshot = Column(String)
  tags = relationship("TagModel", secondary=Link_Tag_Table)



from pydantic import BaseModel 
from sqlalchemy import Column, String, Integer

from poindexter.models import Base


class PluginOutputCreate(BaseModel):
  path: str


class PluginOutput(PluginOutputCreate):
  id: int
  path: str
  class Config:
    orm_mode = True


class PluginOutputModel(Base):
  __tablename__ = 'outputs'
  id = Column(Integer, primary_key=True, index=True)
  path = Column(String)



from enum import Enum

import sqlalchemy

from pydantic import BaseModel
from sqlalchemy import Column, String, Integer

from poindexter.models import Base


class TagType(Enum):
  GOOD = "GOOD"
  BAD = "BAD"
  NEUTRAL = "NEUTRAL"


class TagCreate(BaseModel):
  name: str
  type: TagType
  description: str
  source: str


class Tag(TagCreate):
  id: int
  class Config:
    orm_mode = True


class TagModel(Base):
  __tablename__ = 'tags'
  id = Column(Integer, primary_key=True, index=True)
  name = Column(String)
  type = Column(sqlalchemy.Enum(TagType), default=TagType.NEUTRAL)
  description = Column(String)
  source = Column(String)



from pydantic import BaseModel 
from sqlalchemy import Column, String, Integer

from poindexter.models import Base


class APIKeyCreate(BaseModel):
  name: str
  value: str


class APIKey(APIKeyCreate):
  id: int
  class Config:
    orm_mode = True


class APIKeyModel(Base):
  __tablename__ = 'keys'
  id = Column(Integer, primary_key=True, index=True)
  name = Column(String)
  value = Column(String)



from enum import Enum 
from typing import List

import sqlalchemy

from pydantic import BaseModel 
from sqlalchemy import Column, String, Integer, ForeignKey, Table
from sqlalchemy.orm import relationship

from poindexter.models import Base
from poindexter.models.Tag import Tag


Domain_Tag_Table = Table(
  'domain_tag_association',
  Base.metadata,
  Column('domain_id', ForeignKey('domains.id')),
  Column('tag_id', ForeignKey('tags.id'))
)

class DomainCategory(Enum):
  ANALYSIS = "ANALYSIS"
  CLOUD = "CLOUD"
  COMMUNICATION = "COMMUNICATION"
  DEVELOPMENT = "DEVELOPMENT"
  DOCUMENTATION = "DOCUMENTATION"
  EDUCATION = "EDUCATION"
  FINANCE = "FINANCE"
  FORMS = "FORMS"
  ORGANIZATION = "ORGANIZATION"
  OTHER = "OTHER"
  REMOTE = "REMOTE"
  SHORTNER = "SHORTNER"
  SOCIAL = "SOCIAL"
  STORAGE = "STORAGE"


class DomainCreate(BaseModel):
  name: str
  description: str
  category: DomainCategory


class Domain(DomainCreate):
  id: int
  tags: List[Tag]
  class Config:
    orm_mode = True


class DomainModel(Base):
  __tablename__ = 'domains'
  id = Column(Integer, primary_key=True, index=True)
  name = Column(String)
  description = Column(String)
  category = Column(sqlalchemy.Enum(DomainCategory))
  tags = relationship("TagModel", secondary=Domain_Tag_Table)



from typing import List

from pydantic import BaseModel 
from sqlalchemy import Column, String, Integer, Table, ForeignKey
from sqlalchemy.orm import relationship

from poindexter.models import Base
from poindexter.models.Keyword import Keyword


Profile_Keyword_Table = Table(
  'profile_keyword_association',
  Base.metadata,
  Column('profile_id', ForeignKey('profiles.id')),
  Column('keyword_id', ForeignKey('keywords.id'))
)


class ProfileCreate(BaseModel):
  name: str


class Profile(ProfileCreate):
  id: int
  keywords: List[Keyword]
  class Config:
    orm_mode = True


class ProfileModel(Base):
  __tablename__ = 'profiles'
  id = Column(Integer, primary_key=True, index=True)
  name = Column(String)
  keywords = relationship("KeywordModel", secondary=Profile_Keyword_Table)



from typing import List

from pydantic import BaseModel 
from sqlalchemy import Column, String, Integer, ForeignKey, Table
from sqlalchemy.orm import relationship

from poindexter.models import Base
from poindexter.models.Tag import Tag


Plugin_Tag_Table = Table(
  'plugin_tag_association',
  Base.metadata,
  Column('plugin_id', ForeignKey('plugins.id')),
  Column('tag_id', ForeignKey('tags.id'))
)


class PluginCreate(BaseModel):
  name: str
  description: str
  author: str
  version: str


class Plugin(PluginCreate):
  id: int
  tags: List[Tag]
  class Config:
    orm_mode = True


class PluginModel(Base):
  __tablename__ = 'plugins'
  id = Column(Integer, primary_key=True, index=True)
  name = Column(String)
  description = Column(String)
  author = Column(String)
  version = Column(String)
  tags = relationship("TagModel", secondary=Plugin_Tag_Table)
